# executes a simulation as a series of fixed-length epochs

from neuron import h, gui

EPOCHDUR = 20 # ms length of each epoch in ms
NUM = 3 # number of epochs

##### model cell

soma = h.Section(name = 'soma')
soma.L = 10
soma.diam = 10/h.PI # surface area 100 um2
soma.insert('hh')

##### instrumentation

h.load_file('pyrig.ses') # RunControl, plot of v vs. t in medium wide black
# and IClamp to drive spiking

h.tstop = 60 # ms
h.tstop_changed()

icl = h.IClamp[0]
icl.delay = 1 # ms
icl.dur = 1e9 # ms
icl.amp = 8e-3 # nA

g = h.Graph[0] # the Graph created by rig.ses

h.load_file("pysyndrive.ses") # NetStim and ExpSyn for synaptic drive
ns = h.NetStim[0]
ns.interval = 3*h.PI # ms
ns.number = 1e9
ns.start = 1 # ms
ns.noise = 0 # fully determinsitic
syn = h.ExpSyn[0]
syn.tau = 3 # ms, too fast for a real inh syn, but this is just a demo
nc = h.NetCon(ns, syn)
nc.weight[0] = -2e-4 # uS

## capture time course of soma.v(0.5)

vvec = h.Vector()
vvec.record(soma(0.5)._ref_v)
tvec = h.Vector()
tvec.record(h._ref_t)

##### simulation flow control

## whatever initialization the model requires
def myinit():
  # in this case, simply
  h.finitialize(-65)

## set up mechanism for saving data

tstore = h.Vector()
vstore = h.Vector()

def prepdatastores():
  tstore = h.Vector()
  vstore = h.Vector()

## append or write most recent epoch's data to output files
def stowdata():
  # prevent duplication of the values from the end of the previous epoch
  if (tstore.size > 0):
    vstore.resize(tstore.size - 1)
    tstore.resize(tstore.size - 1)
  #
  tstore.append(tvec)
  vstore.append(vvec)

## update t axis of runtime-system-managed graphs
def updategraph():
  h.tstop = h.t
  h.tstop_changed() 

def housekeeping():
  updategraph()
  stowdata()
  h.frecord_init()

## executes a simulation as a series of fixed-length epochs
## EPOCHDUR specifies length of each epoch in ms
## NUM specifies the number of intervals

def erun():
  prepdatastores()
  h.tstop = EPOCHDUR
  h.tstop_changed() 
  myinit() # model-specific initialization
  for ii in range(NUM):
    h.continuerun(h.t + EPOCHDUR)
    housekeeping() # stuff to do at end of each epoch

erun()

## vstore, tstore contain the concatenated epoch data

##### test by comparing with results of a single run of NUM*EPOCHDUR ms

h.tstop = NUM*EPOCHDUR
h.tstop_changed()
h.run()

##### tests

## visual test
vstore.plot(g, tstore, 2, 1) ## normal width red plot of concatenated epoch data
## on top of medium wide black plot of single run data
g.label(0.434505, 0.392971, "concatenated epochs", 2, 1, 0, 0, 2)
